@@ -37,12 +37,13 @@
 #include <dirent.h>
 #include <magic.h>
 #include <limits.h>
+#include <ctype.h>
 
 /**
  * Number of threads to run in the thread pool.  Should (roughly) match
  * the number of cores on your system.
  */
-#define NUMBER_OF_THREADS 8
+#define NUMBER_OF_THREADS 4
 
 /**
  * How many bytes of a file do we give to libmagic to determine the mime type?
@@ -280,7 +281,7 @@ list_directory (struct ResponseDataContext *rdc,
       rdc->off += snprintf (&rdc->buf[rdc->off], 
 			    rdc->buf_len - rdc->off,
 			    "<li><a href=\"/%s\">%s</a></li>\n",
-			    de->d_name,
+			    fullname,
 			    de->d_name);
     }
   (void) closedir (dir);
@@ -361,6 +362,9 @@ update_directory ()
 					      rdc.buf,
 					      MHD_RESPMEM_MUST_FREE);
   mark_as_html (response);
+  (void) MHD_add_response_header (response,
+				  MHD_HTTP_HEADER_CONNECTION,
+				  "close");
   update_cached_response (response);
 }
 
@@ -472,6 +476,7 @@ process_upload_data (void *cls,
 		     size_t size)
 {
   struct UploadContext *uc = cls;
+  int i;
 
   if (0 == strcmp (key, "category"))
     return do_append (&uc->category, data, size);
@@ -529,7 +534,10 @@ process_upload_data (void *cls,
 		"%s/%s/%s",
 		uc->language,
 		uc->category,
-		filename);      
+		filename); 
+      for (i=strlen (fn)-1;i>=0;i--)
+	if (! isprint ((int) fn[i]))
+	  fn[i] = '_';
       uc->fd = open (fn, 
 		     O_CREAT | O_EXCL 
 #if O_LARGEFILE
@@ -776,6 +784,42 @@ generate_page (void *cls,
 }
 
 
+/**
+ * Function called if we get a SIGPIPE. Does nothing.
+ *
+ * @param sig will be SIGPIPE (ignored)
+ */
+static void
+catcher (int sig)
+{
+  /* do nothing */
+}
+
+
+/**
+ * setup handlers to ignore SIGPIPE.
+ */
+#ifndef MINGW
+static void
+ignore_sigpipe ()
+{
+  struct sigaction oldsig;
+  struct sigaction sig;
+
+  sig.sa_handler = &catcher;
+  sigemptyset (&sig.sa_mask);
+#ifdef SA_INTERRUPT
+  sig.sa_flags = SA_INTERRUPT;  /* SunOS */
+#else
+  sig.sa_flags = SA_RESTART;
+#endif
+  if (0 != sigaction (SIGPIPE, &sig, &oldsig))
+    fprintf (stderr,
+             "Failed to install SIGPIPE handler: %s\n", strerror (errno));
+}
+#endif
+
+
 /**
  * Entry point to demo.  Note: this HTTP server will make all
  * files in the current directory and its subdirectories available
@@ -799,6 +843,9 @@ main (int argc, char *const *argv)
 	       "%s PORT\n", argv[0]);
       return 1;
     }
+  #ifndef MINGW
+  ignore_sigpipe ();
+  #endif
   magic = magic_open (MAGIC_MIME_TYPE);
   (void) magic_load (magic, NULL);
 
@@ -816,12 +863,18 @@ main (int argc, char *const *argv)
 							     MHD_RESPMEM_PERSISTENT);
   mark_as_html (internal_error_response);
   update_directory ();
-  d = MHD_start_daemon (MHD_USE_SELECT_INTERNALLY,
+  d = MHD_start_daemon (MHD_USE_SELECT_INTERNALLY | MHD_USE_DEBUG
+#if EPOLL_SUPPORT 
+			| MHD_USE_EPOLL_LINUX_ONLY
+#endif
+			,
                         port,
                         NULL, NULL, 
 			&generate_page, NULL, 
-			MHD_OPTION_CONNECTION_MEMORY_LIMIT, (size_t) (256 * 1024),
+			MHD_OPTION_CONNECTION_MEMORY_LIMIT, (size_t) (256 * 1024), 
+#if PRODUCTION
 			MHD_OPTION_PER_IP_CONNECTION_LIMIT, (unsigned int) (64),
+#endif
 			MHD_OPTION_CONNECTION_TIMEOUT, (unsigned int) (120 /* seconds */),
 			MHD_OPTION_THREAD_POOL_SIZE, (unsigned int) NUMBER_OF_THREADS,
 			MHD_OPTION_NOTIFY_COMPLETED, &response_completed_callback, NULL,
