@@ -120,7 +120,8 @@ struct Proxy
 	int id;
   int32_t stream_id;
 	bool done;
-	bool error;
+	bool http_error;
+	bool spdy_error;
   bool http_active;
   bool spdy_active;
   bool receiving_done;
@@ -159,10 +160,22 @@ struct global_options
   bool verbose;
   bool only_proxy;
   bool spdy_data_received;
+  bool statistics;
 }
 glob_opt;
 
 
+struct global_statistics
+{
+  //unsigned long long http_bytes_sent;
+  //unsigned long long http_bytes_received;
+  unsigned long long spdy_bytes_sent;
+  unsigned long long spdy_bytes_received;
+  unsigned long long spdy_bytes_received_and_dropped;
+}
+glob_stat;
+
+
 //forbidden headers
 #define SPDY_HTTP_HEADER_TRANSFER_ENCODING "transfer-encoding"
 #define SPDY_HTTP_HEADER_PROXY_CONNECTION "proxy-connection"
@@ -241,6 +254,13 @@ glob_opt;
 	while(0)
   
   
+#define UPDATE_STAT(stat, value) do{\
+  if(glob_opt.statistics)\
+  {\
+    stat += value;\
+  }\
+  }\
+  while(0)
 
 
 void
