@@ -199,6 +199,9 @@ static ssize_t send_callback(spdylay_session *session,
                              const uint8_t *data, size_t length, int flags,
                              void *user_data)
 {
+  (void)session;
+  (void)flags;
+  
   struct Connection *connection;
   ssize_t rv;
   connection = (struct Connection*)user_data;
@@ -237,6 +240,9 @@ static ssize_t recv_callback(spdylay_session *session,
                              uint8_t *buf, size_t length, int flags,
                              void *user_data)
 {
+  (void)session;
+  (void)flags;
+  
   struct Connection *connection;
   ssize_t rv;
   connection = (struct Connection*)user_data;
@@ -278,6 +284,8 @@ static void before_ctrl_send_callback(spdylay_session *session,
                                       spdylay_frame *frame,
                                       void *user_data)
 {
+  (void)user_data;
+  
   if(type == SPDYLAY_SYN_STREAM) {
     struct Request *req;
     int stream_id = frame->syn_stream.stream_id;
@@ -293,6 +301,8 @@ static void on_ctrl_send_callback(spdylay_session *session,
                                   spdylay_frame_type type,
                                   spdylay_frame *frame, void *user_data)
 {
+  (void)user_data;
+  
   char **nv;
   const char *name = NULL;
   int32_t stream_id;
@@ -318,6 +328,8 @@ static void on_ctrl_recv_callback(spdylay_session *session,
                                   spdylay_frame_type type,
                                   spdylay_frame *frame, void *user_data)
 {
+  (void)user_data;
+  
   struct Request *req;
   char **nv;
   const char *name = NULL;
@@ -361,6 +373,9 @@ static void on_stream_close_callback(spdylay_session *session,
                                      spdylay_status_code status_code,
                                      void *user_data)
 {
+  (void)status_code;
+  (void)user_data;
+  
   struct Request *req;
   req = spdylay_session_get_stream_user_data(session, stream_id);
   if(req) {
@@ -383,6 +398,9 @@ static void on_data_chunk_recv_callback(spdylay_session *session, uint8_t flags,
                                         const uint8_t *data, size_t len,
                                         void *user_data)
 {
+  (void)flags;
+  (void)user_data;
+  
   struct Request *req;
   req = spdylay_session_get_stream_user_data(session, stream_id);
   if(req) {
@@ -743,8 +761,19 @@ standard_request_handler(void *cls,
                         const char *version,
                         const char *host,
                         const char *scheme,
-						struct SPDY_NameValue * headers)
+						struct SPDY_NameValue * headers,
+            bool more)
 {
+	(void)cls;
+	(void)request;
+	(void)priority;
+	(void)host;
+	(void)scheme;
+	(void)headers;
+	(void)method;
+	(void)version;
+	(void)more;
+  
 	struct SPDY_Response *response=NULL;
 	
 	if(strcmp(CLS,cls)!=0)
@@ -818,9 +847,6 @@ childproc(int port)
 	if(NULL == (rcvbuf = malloc(strlen(RESPONSE_BODY)+1)))
 		killparent(parent,"no memory");
 
-  SSL_load_error_strings();
-  SSL_library_init();
-
   rv = parse_uri(&uri, uristr);
   if(rv != 0) {
     killparent(parent,"parse_uri failed");
@@ -912,7 +938,7 @@ parentproc( int port)
 	return WEXITSTATUS(childstatus);
 }
 
-int main(int argc, char **argv)
+int main()
 {
 	int port = get_port(12123);
 	parent = getpid();
